<?php

namespace Teknisa\FIN\Modules\ContaPagar\Controllers;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIN\Modules\ContaPagar\Services\RetornoPagarService;

class RetornoPagarController extends TeknisaCrud{

    protected $dataSourceName;
    protected $parameterBag;
    protected $crudService;
    protected $services;
    protected $parameterBagColumns = array("CDEMPRESA", "NRCNABPAG", "CDFILIAL", "NRLANBOLPAG", "VRLIQINI", "VRLIQFIN", "TVALOR", "NRNOTAFISCINI", "NRNOTAFISCFIN", "TNOTA");

    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, RetornoPagarService $services)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->services = $services;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;
        parent::find($request, $response);
    }

    public function carregaRetorno(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $dataSet = $this->services->carregaRetorno($row);
            $response->addDataSet(new DataSet("BOLETOS", $dataSet[0]));
            $response->addDataSet(new DataSet("OCORRENCIAS", $dataSet[1]));
            $response->addDataSet(new DataSet("SITUACAOCNAB", $dataSet[2]));
        }catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function insereRetorno(DTO\Request $request, DTO\Response $response){
        try {
            // Timeout - Ilimitado
            set_time_limit(0);
            
            $rows = $request->getDataSet()->getRows();
            $dsarqcnabpag = $rows[0]['DSARQCNABPAG'];
            $cdempresa = $rows[0]['CDEMPRESA'];
            $cdbanco = $rows[0]['CDBANCO'];
            $boletos = $rows[1];
            $ocorrencias = $rows[2];
            $this->services->insereRetorno($boletos, $ocorrencias, $rows[0]['NOMEARQUIVO'], $cdempresa, $cdbanco);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    /**
     * Cancela Título Retorno
     * @param DTO\Request\DataSet $dataset
     * @param DTO\Response $response
     */
    public function cancelaTituloRetorno(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $retorno = $rows[0];
            $boletos = $rows[1];

            // Cancela Título Retorno
            $this->services->cancelaTitulosRetorno($retorno, $boletos);

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
}